package network

import (
	"errors"
	"fmt"

	"github.com/cloudfoundry-incubator/garden"
	"github.com/cloudfoundry-incubator/garden-linux/fences/netfence/network/iptables"
)

type FilterFactory interface {
	Create(id string) Filter
}

type Filter interface {
	NetOut(network string, port uint32, portRange string, protocol garden.Protocol, icmpType int32, icmpCode int32) error
}

type filterFactory struct {
	instancePrefix string
	chainFactory   iptables.ChainFactory
}

type filter struct {
	instanceChain iptables.Chain
}

func NewFilterFactory(tag string, chainFactory iptables.ChainFactory) FilterFactory {
	return &filterFactory{instancePrefix: fmt.Sprintf("w-%s-instance-", tag),
		chainFactory: chainFactory,
	}
}

func (ff *filterFactory) Create(id string) Filter {
	return &filter{instanceChain: ff.chainFactory.CreateChain(ff.instancePrefix + id)}
}

func (fltr *filter) NetOut(network string, port uint32, portRange string, protocol garden.Protocol, icmpType int32, icmpCode int32) error {
	if protocol != garden.ProtocolICMP && (icmpType != -1 || icmpCode != -1) {
		return errors.New("invalid rule: icmp code or icmp type can only be specified with protocol ICMP")
	}
	if network == "" && port == 0 && portRange == "" {
		return errors.New("invalid rule: either network or port (range) must be specified")
	}
	if (port != 0 || portRange != "") && !fltr.protocolAllowsPortFiltering(protocol) {
		return errors.New("invalid rule: a port (range) can only be specified with protocol TCP or UDP")
	}
	if port != 0 && portRange != "" {
		return errors.New("invalid rule: port and port range cannot both be specified")
	}
	return fltr.instanceChain.PrependFilterRule(protocol, network, port, portRange, icmpType, icmpCode)
}

func (fltr *filter) protocolAllowsPortFiltering(protocol garden.Protocol) bool {
	return protocol == garden.ProtocolTCP || protocol == garden.ProtocolUDP
}
